import java.util.*;

public class NewCollectionImpl<E> {
    // Overview: Tipo modificabile che rappresenta una collezione indicizzata di elementi di E
    
    // AF: { <data[i_1], i_1>, ..., <data[i_k], i_k> | 0 <= i_j < dim && data[i_j] != null }
    // IR: dim = data.length >= size >= 0 && data != null
    
    private Object[] data; // array che memorizza gli elementi della collezione
    private int size;      // numero di elementi attualmente contenuti nella collezione
    private int dim;       // dimensione dell'array
    
    public NewCollectionImpl(int lun) {
        data = new Object[lun];
        size = 0; dim = lun;
    }
    
    private int emptySlot( ) {
        for(int i = dim-1; i >= 0; i--)
            if (data[i] == null) return i;
        
        return -1;
    }

    private void copyArray(int dimNew) {
        Object[] dataNew = new Object[dimNew];

        for(int i = 0; i<dim; i++)
            dataNew[i] = data[i];
        
        data = dataNew;
        dim = dimNew;
    }
        
    /* inserisce l'elemento elem nella collezione in posizione p,
       spostando l'elemento contenuto in precedenza in una posizione libera */
    public void insertAt(E elem, int p) throws NullPointerException, OutOfBoundsException {
        if (elem == null) throw new NullPointerException();
        if (p < 0) throw new OutOfBoundsException();
        
        if (dim <= p)           // crea un array di dimensione p+1
            copyArray(p+1);     // era accettabile la soluzione che lanciava una eccezione

        if (dim == size)            // crea un array di dimensione dim+1
            copyArray(dim+1);       // era accettabile la soluzione che lanciava una eccezione
                                    // (dimensione costante) o perdeva il dato in p sovrascrivendolo
        if (data[p] == null)
            data[p] = elem;
        else {
            data[emptySlot()] = data[p];
            data[p] = elem;
        }

        size++;
    }
    /*
     MODIFIES: this
     EFFECTS: se elem != null e data[p] == null inserisce elem nella collezione in posizione p
     (eventualmente aumentando la dimensione di data), se p < 0 lancia una OutOfBoundsException
     e se elem == null una NullPointerException (eccezioni disponibili in Java, unchecked),
     se la posizione p era gia' occupata sposta data[p] nella prima posizione libera
     */
    
    /* restituisce l'elemento in posizione p nella collezione */
    public E get(int p) throws OutOfBoundsException {
        if (p < 0) throw new OutOfBoundsException();

        if (dim <= p) return null;
        else return (E) data[p];
    }
    /*
     EFFECTS: se dim > p restituisce data[p] (che puo' essere null), altrimenti null
     */
    
    
    /* restituisce la posizione della prima occorrenza dell'elemento elem,
       altrimenti -1 se elem non appartiene alla collezione */
    public int indexOf(E elem) throws NullPointerException {
        if (elem == null) throw new NullPointerException();
        
        for(int i = 0; i<dim; i++)
            if (elem.equals(data[i]))
                return i;
        
        return -1;
    }
    /*
     EFFECTS: se elem != null e occorre in data restituisce il piu' piccolo i tale che elem.equals(data[i]),
     se elem == null lancia una NullPointerException (eccezione disponibile in Java, unchecked),
     se elem non occorre in data restituisce -1
     */
}